
extern crate termion;
use termion::{color, clear, cursor, style};
use termion::raw::IntoRawMode;
use std::io::{Write, stdout};
use std::thread;
use std::time::Duration;

#[path = "../src/sensor.rs"]
mod sensor;
use crate::sensor::Sensor;

pub fn print_position(s: &Sensor, current_distance: u16, pos: u16) {
    let mut stdout = stdout().into_raw_mode().unwrap();

    print_map(s); 
    print_x(s, pos);
    print_x_history(s, pos);
    print_block_num(s, pos);
    print_pos_distances(s, current_distance, pos);
    print_end_clear(s);

    stdout.flush();

    print_delay(s.gui.print_delay); 
}

pub fn init_map(s: &Sensor) {
    let mut stdout = stdout().into_raw_mode().unwrap(); 
    let left_text = String::from("0m |");
    let right_text = String::from(format!("| {}m", (s.max_distance as f32/1000.0)));

    write!(stdout, "{clear}{goto_l}{}{}{goto_r}{}",
           left_text,
           " ".repeat(s.gui.max_blocks as usize),
           right_text,
           clear = clear::All,
           goto_l = cursor::Goto(s.gui.x-left_text.len() as u16, s.gui.y),
           goto_r = cursor::Goto(s.gui.x+s.gui.max_blocks, s.gui.y)).unwrap();

    for step in 0..s.gui.max_blocks {
        write!(stdout, "{goto_blocks}{}{goto_num}{}", 
                 "=",
                 (step+1).to_string() + " blocks",
                 goto_blocks = cursor::Goto(s.gui.x+step, s.gui.y), 
                 goto_num = cursor::Goto(s.gui.x, s.gui.y-2)).unwrap();
        stdout.flush().unwrap();
        print_delay(10);
    }
}
pub fn print_map(s: &Sensor) { 
    let mut stdout = stdout().into_raw_mode().unwrap();
    
    write!(stdout, "{goto}{col}{}", 
             "=".repeat(s.gui.max_blocks as usize),
             col = color::Fg(color::Green),
             goto = cursor::Goto(s.gui.x,s.gui. y)).unwrap();
    stdout.flush();
}

fn print_delay(delay: u64) {
    thread::sleep(Duration::from_millis(delay));
}

pub fn print_x(s: &Sensor, pos: u16) {
    let mut stdout = stdout().into_raw_mode().unwrap();
    write!(stdout, "\r{goto}{col}x{reset}",
             goto = cursor::Goto(pos+s.gui.x-1, s.gui.y),
             col = color::Fg(color::Red),
             reset = color::Fg(color::Reset)).unwrap();  
}

pub fn print_x_history(s: &Sensor, pos: u16){
    let mut stdout = stdout().into_raw_mode().unwrap();
    write!(stdout, "\r{goto}{col}x{reset}",
             goto = cursor::Goto(s.gui.x+pos-1, s.gui.y+1),
             col = color::Fg(color::Yellow),
             reset = color::Fg(color::Reset)).unwrap(); 
}

pub fn print_block_num(s: &Sensor, pos: u16) {
    let mut stdout = stdout().into_raw_mode().unwrap();
    write!(stdout, "{goto}{clear}{col}{}{reset}",
             pos,
             goto = cursor::Goto(s.gui.x+pos-1, s.gui.y-1),
             col = color::Fg(color::LightCyan),
             clear = clear::CurrentLine,
             reset = color::Fg(color::Reset)).unwrap(); 
}

fn print_pos_distances(s: &Sensor, current_distance: u16, pos: u16) {
    let mut stdout = stdout().into_raw_mode().unwrap(); 

    let d = get_distance(current_distance as f32);
    let (mm, cm, dm, m) = (d.0, d.1, d.2,d.3);
 
    if (mm as u16) < s.warning_distance {
        write!(stdout, "{goto}{clear}{col}{}{reset}",
               "COLLISSION IMMINENT!",
               col = color::Fg(color::Red),
               clear = clear::CurrentLine,
               reset = color::Fg(color::Reset),
               goto = cursor::Goto(s.gui.x, s.gui.y-5)).unwrap();
    }
    else {
        write!(stdout, "{goto}{clear}{col}{}{reset}",
               "YEAAAAAAH BOIIIIIII",
               clear = clear::CurrentLine,
               col = color::Fg(color::Green),
               reset = color::Fg(color::Reset),
               goto = cursor::Goto(s.gui.x, s.gui.y-5)).unwrap();
    }

    write!(stdout, "{goto}{clear}{col}{}mm | {:.3}cm | {:.3}dm | {:.4}m{reset}",
           mm.to_string(),
           cm.to_string(),
           dm.to_string(),
           m.to_string(),
           clear = clear::CurrentLine,
           col = color::Fg(color::Yellow),
           reset = color::Fg(color::Reset),
           goto = cursor::Goto(s.gui.x, s.gui.y-4)).unwrap();
}

fn get_distance(current_distance: f32) -> (f32, f32, f32, f32) {
    let mm = (current_distance as f32);
    let cm = (current_distance/10.0 as f32);
    let dm = (current_distance/100.0 as f32);
    let m  = (current_distance/1000.0 as f32);
    (mm, cm, dm, m)
}


fn print_end_clear(s: &Sensor) {
    let mut stdout = stdout().into_raw_mode().unwrap(); 
    write!(stdout, "{goto}{clear}",
           clear = clear::CurrentLine,
           goto = cursor::Goto(1, s.gui.y+10)).unwrap();

}


//////////////////////////////////////////////////////////////////////
pub fn print_distance(s: &Sensor, current_distance: u16) {
    let mut stdout = stdout().into_raw_mode().unwrap();

    let mm_per_block: f32 = s.max_distance as f32 / s.gui.max_blocks as f32 ; //6500/65=100, that is: 100mm per block
    let block_pos: u16 = (0.5 + (current_distance as f32/ mm_per_block)) as u16;

    if block_pos > s.gui.max_blocks || block_pos < s.gui.min_blocks{
        write!(stdout, "{goto}{col}Invalid Allowed Distance>>> Expected: 0-{}mm | Actual: {}mm{reset}{goto_panic}\n\n\n", 
               s.max_distance,
               current_distance,
               goto = cursor::Goto(s.gui.x, s.gui.y+3),
               goto_panic = cursor::Goto(0, s.gui.y+5),
               col = color::Fg(color::Red),
               reset = color::Fg(color::Reset)).unwrap();
        //panic!(format!("A value that is not deemed allowed was received: \
        //{}mm", current_distance));
        
        //Clears the GUI Position and Block Num
        clear_position_data(&s, current_distance, block_pos);
    }
    else {
        write!(stdout, "{goto}{clear}Distance: {}mm, Block: {} ",
               current_distance, 
               block_pos,
               clear = clear::CurrentLine,
               goto = cursor::Goto(s.gui.x, s.gui.y+3)).unwrap();
    
        print_position(s, current_distance, block_pos);
    }
     
}


fn clear_position_data(s: &Sensor, current_distance: u16, pos: u16) {
    let mut stdout = stdout().into_raw_mode().unwrap();
    print_map(&s);
    //print_pos_distances(s, current_distance, pos);
    write!(stdout, "{goto}{clear}",
         goto = cursor::Goto(s.gui.x-1, s.gui.y-1),
         clear = clear::CurrentLine);
    write!(stdout, "{goto}{clear}{col}{}{reset}",
           "ERROR: INVALID RANGE DATA",
           col = color::Fg(color::Red),
           clear = clear::CurrentLine,
           reset = color::Fg(color::Reset),
           goto = cursor::Goto(s.gui.x, s.gui.y-5)).unwrap();
    write!(stdout, "{goto}{clear}",
           clear = clear::CurrentLine,
           goto = cursor::Goto(s.gui.x, s.gui.y-4)).unwrap();
    print_end_clear(&s); 
}
